/*
 * file CqDb.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqDb
 *
 *  Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stpex.StpExBase;

/**
 * An interface for accessing administrative information regarding a
 * login to a ClearQuest database; common to both db-sets and user databases.
 */
public interface CqDb extends CqResource
{
    /**
     * Returns the string that the current user enters as the login name when
     * authenticating. The return value may be different from a Rational
     * ClearQuest user name if the user is LDAP authenticated. Use the
     * CURRENT_USER property to get the Rational ClearQuest name of the user
     * stored in the user profile record for the user. Returns the
     * authenticating name used to access this database resource. The value
     * returned is the name used to authenticate the user, not the Rational
     * ClearQuest user login field name that is stored in the user profile
     * record for the user. The return value may be an LDAP login name (for
     * example, myname@us.ibm.com) and not a Rational ClearQuest user name (for
     * example, mycqname).
     */
    PropertyName<String> AUTHENTICATION_LOGIN_NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE,
                                 "authentication-login-name");

    /**
     * Returns the value of the
     * {@link #AUTHENTICATION_LOGIN_NAME AUTHENTICATION_LOGIN_NAME} property as
     * defined by this proxy.
     * 
     * @return The authentication login name. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #AUTHENTICATION_LOGIN_NAME AUTHENTICATION_LOGIN_NAME}
     *             property.
     */
    String getAuthenticationLoginName() throws WvcmException;

    /**
     * Returns the CqUser structure in this database that corresponds to the
     * logged-in user identified by the host provider.
     */
    PropertyName<CqUser> CURRENT_USER =
        new PropertyName<CqUser>(PROPERTY_NAMESPACE, "current-user");

    /**
     * Returns the value of the {@link #CURRENT_USER CURRENT_USER} property as
     * defined by this proxy.
     * 
     * @return The CqUser object for the user identified by this proxy's
     *         provider. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CURRENT_USER CURRENT_USER} property.
     */
    CqUser getCurrentUser() throws WvcmException;

    /**
     * Returns the name of the replica hosting this database.
     */
    PropertyName<CqReplica> LOCAL_REPLICA =
        new PropertyName<CqReplica>(PROPERTY_NAMESPACE, "local-replica");

    /**
     * Returns the value of the {@link #LOCAL_REPLICA LOCAL_REPLICA} property as
     * defined by this proxy.
     * 
     * @return A CqReplica proxy for the local replica. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #LOCAL_REPLICA LOCAL_REPLICA} property.
     */
    CqReplica getLocalReplica() throws WvcmException;

    /**
     * A list of all groups known to this database.
     */
    PropertyName<ResourceList<CqGroup>> ALL_GROUPS =
        new PropertyName<ResourceList<CqGroup>>(PROPERTY_NAMESPACE,
                                                "all-groups");

    /**
     * Returns the value of the {@link #ALL_GROUPS ALL_GROUPS} property as
     * defined by this proxy.
     * 
     * @return The value. A list of CqGroup proxies identifying all the
     *         groups known to the database. Will never be <b>null</b> but
     *         the list may be empty.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_GROUPS ALL_GROUPS} property.
     */
    ResourceList<CqGroup> getAllGroups() throws WvcmException;

    /**
     * A list of all users known to this database.
     */
    PropertyName<ResourceList<CqUser>> ALL_USERS =
        new PropertyName<ResourceList<CqUser>>(PROPERTY_NAMESPACE, "all-users");

    /**
     * Returns the value of the {@link #ALL_USERS ALL_USERS} property as
     * defined by this proxy.
     * 
     * @return The value. A list of CqUser proxies identifying all the users
     *         known to this database. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_USERS ALL_USERS} property.
     */
    ResourceList<CqUser> getAllUsers() throws WvcmException;

    /**
     * The name of the distinguished group to which every user implicitly
     * belongs.
     */
    PropertyName<String> EVERYONE_GROUP_NAME =
        new PropertyName<String>(StpExBase.PROPERTY_NAMESPACE,
                                 "everyone-group-name");

    /**
     * Returns the value of the
     * {@link #EVERYONE_GROUP_NAME EVERYONE_GROUP_NAME} property as defined
     * by this proxy.
     * 
     * @return The name of the group of which every user is implicitly a
     *         member. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #EVERYONE_GROUP_NAME EVERYONE_GROUP_NAME}
     *             property.
     */
    String getEveryoneGroupName() throws WvcmException;

    /**
     * The name of the CqUser property that is used for correlating LDAP user
     * records to Rational ClearQuest user records; <b>null</b> if the mapping
     * is not configured. May return an exception if there is an error
     * connecting to the db-set associated with this database (that is, the
     * master database or schema repository). In a MultiSite environment,
     * Rational ClearQuest enforces that the same LDAP_PROPERTY is used for all
     * sites (that is, the same Rational ClearQuest User field), but allows that
     * the LDAP attribute that is mapped may be site specific. This allows you
     * to have different LDAP schemas at different sites, but allows Rational
     * ClearQuest to enforce the uniqueness of the Rational ClearQuest mapping
     * field values across an entire database set. LDAP_PROPERTY is <b>null</b>
     * if the database set has not been configured for LDAP use.
     */
    PropertyName<PropertyName<String>> LDAP_PROPERTY =
        new PropertyName<PropertyName<String>>(PROPERTY_NAMESPACE,
                                               "ldap-property");
    /**
     * Returns the value of the {@link #LDAP_PROPERTY LDAP_PROPERTY} property as
     * defined by this proxy.
     * 
     * @return A PropertyName identifying the property of CqUser resources that
     *         is used for LDAP authentication or <b>null</b> if LDAP mapping
     *         has not been configured for this database. The possible
     *         PropertyName values are: CqUser.PHONE, CqUser.FULL_NAME,
     *         CqUser.EMAIL, CqUser.MISCELLANEOUS_INFORMATION, and
     *         CqUser.LOGIN_NAME.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #LDAP_PROPERTY LDAP_PROPERTY} property.
     */
    PropertyName<String> getLDAPProperty() throws WvcmException;

    PropertyName<String[]> DIAG_INFO = 
    new PropertyName<String[]>(PROPERTY_NAMESPACE, "diag-info");

    /**
     * Returns the value of the {@link #DIAG_INFO DIAG_INFO} property as defined
     * by this proxy.
     * 
     * @return A String[] containing the current settings for the generation of
     *         diagnostic information by the ClearQuest application. Will never
     *         be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DIAG_INFO DIAG_INFO} property.
     */
    String[] getDiagInfo() throws WvcmException;


    /**
     * Defines a new value for the {@link #DIAG_INFO DIAG_INFO} property of this
     * proxy.
     * 
     * @param info A String[] containing the new diagnostic settings.
     */
    void setDiagInfo(String[] info);


    /**
     * Verifies that the provided login name and password are valid and map to
     * the current user.
     * 
     * @param loginName The user's login name
     * @param password The password associated with the loginName
     * @param feedback A Feedback object in which may be specified the CqUser
     *            properties to be returned in the result proxy.
     * 
     * @return The CqUser to which the specified credentials map if the user is
     *         successfully validated; <b>null</b> otherwise.
     */
    CqUser doValidateUserCredentials(String loginName,
                                     String password,
                                     Feedback feedback) throws WvcmException;
}
